@@ -7,8 +7,7 @@
 """
 import abc
 import copy
-from multiprocessing import Process
-
+# from multiprocessing import Process
 import datetime
 import json
 import sys
@@ -38,8 +37,9 @@ from function_scheduling_distributed_framework.utils.bulk_operation import Mongo
 from function_scheduling_distributed_framework.utils.mongo_util import MongoMixin
 
 
-def delete_keys_and_return_new_dict(dictx: dict, keys: list):
+def delete_keys_and_return_new_dict(dictx: dict, keys: list = None):
     dict_new = copy.copy(dictx)  # 主要是去掉一级键 publish_time，浅拷贝即可。
+    keys = ['publish_time', 'publish_time_format', 'extra'] if keys is None else keys
     for dict_key in keys:
         try:
             dict_new.pop(dict_key)
@@ -56,7 +56,12 @@ class ExceptionForRequeue(Exception):
     """框架检测到此错误，重新放回队列中"""
 
 
-def _get_publish_time(paramsx):
+def _get_publish_time(paramsx: dict):
+    """
+    原来存放控制参数的位置没想好，建议所有控制参数放到extra键的字典值里面。
+    :param paramsx:
+    :return:
+    """
     return paramsx.get('extra', {}).get('publish_time', None) or paramsx.get('publish_time', None)
 
 
@@ -72,9 +77,9 @@ class FunctionResultStatus(LoggerMixin, LoggerLevelSetterMixin):
         publish_time = _get_publish_time(params)
         if publish_time:
             self.publish_time_str = time_util.DatetimeConverter(publish_time).datetime_str
-        function_params = delete_keys_and_return_new_dict(params, ['publish_time', 'publish_time_format', 'extra'])
+        function_params = delete_keys_and_return_new_dict(params,)
         self.params = function_params
-        self.params_str = json.dumps(function_params)
+        self.params_str = json.dumps(function_params,ensure_ascii=False)
         self.result = ''
         self.run_times = 0
         self.exception = ''
@@ -105,6 +110,8 @@ class FunctionResultStatus(LoggerMixin, LoggerLevelSetterMixin):
             item.update({'insert_time': datetime.datetime.now(),
                          'utime': datetime.datetime.utcnow(),
                          })
+        else:
+            item = delete_keys_and_return_new_dict(item,['insert_time','utime'])
         item['_id'] = str(uuid.uuid4())
         self.logger.debug(item)
         return item
@@ -189,6 +196,12 @@ class ConsumersManager:
 
 class FunctionResultStatusPersistanceConfig(LoggerMixin):
     def __init__(self, is_save_status: bool, is_save_result: bool, expire_seconds: int):
+        """
+        :param is_save_status:
+        :param is_save_result:
+        :param expire_seconds: 设置统计的过期时间，在mongo里面自动会移除这些过期的执行记录。
+        """
+
         if not is_save_status and is_save_result:
             raise ValueError(f'你设置的是不保存函数运行状态但保存函数运行结果。不允许你这么设置')
         self.is_save_status = is_save_status
@@ -239,7 +252,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
     # noinspection PyProtectedMember
     def __init__(self, queue_name, *, consuming_function: Callable = None, function_timeout=0,
-                 threads_num=50, specify_threadpool=None, concurrent_mode=1,
+                 threads_num=50,concurrent_num = 50,specify_threadpool=None, concurrent_mode=1,
                  max_retry_times=3, log_level=10, is_print_detail_exception=True,
                  msg_schedule_time_intercal=0.0, qps=0, msg_expire_senconds=0,
                  logger_prefix='', create_logger_file=True, do_task_filtering=False,
@@ -247,12 +260,15 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                  is_do_not_run_by_specify_time_effect=False,
                  do_not_run_by_specify_time=('10:00:00', '22:00:00'),
                  schedule_tasks_on_main_thread=False,
-                 function_result_status_persistance_conf=FunctionResultStatusPersistanceConfig(False, False, 7 * 24 * 3600)):
+                 function_result_status_persistance_conf=FunctionResultStatusPersistanceConfig(
+                     False, False, 7 * 24 * 3600),
+                 is_using_rpc_mode=False):
         """
         :param queue_name:
         :param consuming_function: 处理消息的函数。
         :param function_timeout : 超时秒数，函数运行超过这个时间，则自动杀死函数。为0是不限制。
         :param threads_num:线程或协程并发数量
+        :param concurrent_num:并发数量，这个覆盖threads_num。以后会废弃threads_num参数，因为表达的意思不太准确，不一定是线程模式并发。
         :param specify_threadpool:使用指定的线程池/携程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
         :param concurrent_mode:并发模式，暂时支持 线程 、gevent、eventlet三种模式。  1线程  2 gevent 3 evenlet
         :param max_retry_times:
@@ -270,8 +286,9 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         :param is_do_not_run_by_specify_time_effect :是否使不运行的时间段生效
         :param do_not_run_by_specify_time   :不运行的时间段
         :param schedule_tasks_on_main_thread :直接在主线程调度任务，意味着不能直接在当前主线程同时开启两个消费者。
-        :function_result_status_persistance_conf   :配置。是否保存函数的入参，运行结果和运行状态到mongodb。这一步用于后续的参数追溯，
-        任务统计和web展示，需要安装mongo。
+        :param function_result_status_persistance_conf   :配置。是否保存函数的入参，运行结果和运行状态到mongodb。
+               这一步用于后续的参数追溯，任务统计和web展示，需要安装mongo。
+        :param is_using_rpc_mode 是否使用rpc模式，可以在发布端获取消费端的结果回调，但消耗一定性能，使用async_result.result时候会等待阻塞住当前线程。
         """
         ConsumersManager.consumers_queue__info_map[queue_name] = current_queue__info_dict = copy.copy(locals())
         current_queue__info_dict['consuming_function'] = str(consuming_function)  # consuming_function.__name__
@@ -297,7 +314,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self.queue_name = queue_name  # 可以换成公有的，免得外部访问有警告。
         self.consuming_function = consuming_function
         self._function_timeout = function_timeout
-        self._threads_num = threads_num
+        self._threads_num = concurrent_num if threads_num == 50 else threads_num  # concurrent参数优先，以后废弃threads_num参数。
         self._specify_threadpool = specify_threadpool
         self._threadpool = None  # 单独加一个检测消息数量和心跳的线程
         self._concurrent_mode = concurrent_mode
@@ -324,7 +341,6 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         sys.stdout.write(f'{time.strftime("%H:%M:%S")} "{current_queue__info_dict["where_to_instantiate"]}"  \033[0;30;44m此行 实例化队列名 {current_queue__info_dict["queue_name"]} 的消费者, 类型为 {self.__class__}\033[0m\n')
 
         self._do_task_filtering = do_task_filtering
-
         self._redis_filter_key_name = f'filter_zset:{queue_name}' if task_filtering_expire_seconds else f'filter_set:{queue_name}'
         filter_class = RedisFilter if task_filtering_expire_seconds == 0 else RedisImpermanencyFilter
         self._redis_filter = filter_class(self._redis_filter_key_name, task_filtering_expire_seconds)
@@ -345,6 +361,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
         self._result_persistence_helper = ResultPersistenceHelper(function_result_status_persistance_conf, queue_name)
 
+        self._is_using_rpc_mode = is_using_rpc_mode
+
         self.stop_flag = False
 
         self._publisher_of_same_queue = None
@@ -359,29 +377,35 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     def custom_init(self):
         pass
 
-    def keep_circulating(self, time_sleep=0.001, exit_if_function_run_sucsess=False, is_display_detail_exception=True):
+    def keep_circulating(self, time_sleep=0.001, exit_if_function_run_sucsess=False, is_display_detail_exception=True, block=True):
         """间隔一段时间，一直循环运行某个方法的装饰器
         :param time_sleep :循环的间隔时间
         :param is_display_detail_exception
         :param exit_if_function_run_sucsess :如果成功了就退出循环
+        :param block:是否阻塞在当前主线程运行。
         """
 
         def _keep_circulating(func):
-            # noinspection PyBroadException
             @wraps(func)
             def __keep_circulating(*args, **kwargs):
-                while 1:
-                    if self.stop_flag:
-                        break
-                    try:
-                        result = func(*args, **kwargs)
-                        if exit_if_function_run_sucsess:
-                            return result
-                    except Exception as e:
-                        msg = func.__name__ + '   运行出错\n ' + traceback.format_exc(limit=10) if is_display_detail_exception else str(e)
-                        self.logger.error(msg)
-                    finally:
-                        time.sleep(time_sleep)
+
+                # noinspection PyBroadException
+                def ___keep_circulating():
+                    while 1:
+                        try:
+                            result = func(*args, **kwargs)
+                            if exit_if_function_run_sucsess:
+                                return result
+                        except Exception as e:
+                            msg = func.__name__ + '   运行出错\n ' + traceback.format_exc(limit=10) if is_display_detail_exception else str(e)
+                            self.logger.error(msg)
+                        finally:
+                            time.sleep(time_sleep)
+
+                if block:
+                    return ___keep_circulating()
+                else:
+                    threading.Thread(target=___keep_circulating, ).start()
 
             return __keep_circulating
 
@@ -389,7 +413,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
     def start_consuming_message(self):
         self.logger.warning(f'开始消费 {self._queue_name} 中的消息')
-        self.threadpool.submit(self.keep_circulating(20)(self.check_heartbeat_and_message_count))
+        self.keep_circulating(20, block=False)(self.check_heartbeat_and_message_count)()
         self._redis_filter.delete_expire_filter_task_cycle()
         if self._schedule_tasks_on_main_thread:
             self.keep_circulating(1)(self._shedual_task)()
@@ -398,10 +422,16 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
     @abc.abstractmethod
     def _shedual_task(self):
+        """
+        每个子类必须实现这个的方法，完成如何从中间件取出消息，并将函数和运行参数添加到工作池。
+        :return:
+        """
         raise NotImplementedError
 
     def _run(self, kw: dict, ):
-        if self._do_task_filtering and self._redis_filter.check_value_exists(kw['body']):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
+        do_task_filtering_priority = self.__get_priority_conf(kw, 'do_task_filtering')
+        function_only_params = delete_keys_and_return_new_dict(kw['body'],)
+        if do_task_filtering_priority and self._redis_filter.check_value_exists(function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
             self.logger.info(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
             self._confirm_consume(kw)
             return
@@ -415,39 +445,48 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 self._current_time_for_execute_task_times_every_minute = time.time()
                 self._execute_task_times_every_minute = 0
         self._run_consuming_function_with_confirm_and_retry(kw, current_retry_times=0, function_result_status=FunctionResultStatus(
-            self.queue_name, self.consuming_function.__name__, kw['body']))
+            self.queue_name, self.consuming_function.__name__, kw['body']), do_task_filtering_priority=do_task_filtering_priority)
 
-    def _run_consuming_function_with_confirm_and_retry(self, kw: dict, current_retry_times, function_result_status: FunctionResultStatus):
-        if current_retry_times < self._max_retry_times:
+    def __get_priority_conf(self, kw: dict, broker_task_config_key: str):
+        broker_task_config = kw['body'].get('extra', {}).get(broker_task_config_key, None)
+        if broker_task_config is None:
+            return getattr(self, f'_{broker_task_config_key}')
+        else:
+            return broker_task_config
+
+    def _run_consuming_function_with_confirm_and_retry(self, kw: dict, current_retry_times, function_result_status: FunctionResultStatus, do_task_filtering_priority):
+        function_only_params = delete_keys_and_return_new_dict(kw['body'])
+        if current_retry_times < self.__get_priority_conf(kw, 'max_retry_times'):
             function_result_status.run_times += 1
             # noinspection PyBroadException
             t_start = time.time()
             try:
-                function_run = self.consuming_function if self._function_timeout == 0 else self._concurrent_mode_dispatcher.timeout_deco(self._function_timeout)(self.consuming_function)
+                function_run = self.consuming_function if self._function_timeout == 0 else self._concurrent_mode_dispatcher.timeout_deco(self.__get_priority_conf(kw, 'function_timeout'))(self.consuming_function)
                 if self._is_consuming_function_use_multi_params:  # 消费函数使用传统的多参数形式
-                    function_result_status.result = function_run(**delete_keys_and_return_new_dict(kw['body'], ['publish_time', 'publish_time_format', 'extra']))
+                    function_result_status.result = function_run(**function_only_params)
                 else:
-                    function_result_status.result = function_run(delete_keys_and_return_new_dict(kw['body'], ['publish_time', 'publish_time_format', 'extra']))  # 消费函数使用单个参数，参数自身是一个字典，由键值对表示各个参数。
+                    function_result_status.result = function_run(function_only_params)  # 消费函数使用单个参数，参数自身是一个字典，由键值对表示各个参数。
                 function_result_status.success = True
                 self._confirm_consume(kw)
-                if self._do_task_filtering:
-                    self._redis_filter.add_a_value(kw['body'])  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+                if do_task_filtering_priority:
+                    self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
                 self.logger.debug(f' 函数 {self.consuming_function.__name__}  '
-                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {kw["body"]} 】。  {ConsumersManager.get_concurrent_info()}')
+                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {ConsumersManager.get_concurrent_info()}')
 
             except Exception as e:
                 if isinstance(e, (PyMongoError, ExceptionForRequeue)):  # mongo经常维护备份时候插入不了或挂了，或者自己主动抛出一个ExceptionForRequeue类型的错误会重新入队，不受指定重试次数逇约束。
                     self.logger.critical(f'函数 [{self.consuming_function.__name__}] 中发生错误 {type(e)}  {e}')
                     return self._requeue(kw)
                 self.logger.error(f'函数 {self.consuming_function.__name__}  第{current_retry_times + 1}次发生错误，'
-                                  f'函数运行时间是 {round(time.time() - t_start, 4)} 秒,\n  入参是 【 {kw["body"]} 】   \n 原因是 {type(e)} {e} ', exc_info=self._is_print_detail_exception)
+                                  f'函数运行时间是 {round(time.time() - t_start, 4)} 秒,\n  入参是 【 {function_only_params} 】   \n 原因是 {type(e)} {e} ', exc_info=self.__get_priority_conf(kw, 'is_print_detail_exception'))
                 function_result_status.exception = f'{e.__class__.__name__}    {str(e)}'
-                self._run_consuming_function_with_confirm_and_retry(kw, current_retry_times + 1, function_result_status)
+                self._run_consuming_function_with_confirm_and_retry(kw, current_retry_times + 1, function_result_status, do_task_filtering_priority)
         else:
-            self.logger.critical(f'函数 {self.consuming_function.__name__} 达到最大重试次数 {self._max_retry_times} 后,仍然失败， 入参是 【 {kw["body"]} 】')
+            self.logger.critical(f'函数 {self.consuming_function.__name__} 达到最大重试次数 {self.__get_priority_conf(kw, "max_retry_times")} 后,仍然失败， 入参是 【 {function_only_params} 】')
             self._confirm_consume(kw)  # 错得超过指定的次数了，就确认消费了。
         self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
-        if kw['body'].get('extra', {}).get('is_using_rpc_mode', False):
+        if self.__get_priority_conf(kw, 'is_using_rpc_mode'):
+            # print(function_result_status.get_status_dict(without_datetime_obj=True))
             RedisMixin().redis_db_frame.lpush(kw['body']['extra']['task_id'], json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
             RedisMixin().redis_db_frame.expire(kw['body']['extra']['task_id'], 600)
 
@@ -476,9 +515,10 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             time.sleep(self.time_interval_for_check_do_not_run_time)
             return
         publish_time = _get_publish_time(kw['body'])
-        if self._msg_expire_senconds != 0 and time.time() - self._msg_expire_senconds > publish_time:
+        msg_expire_senconds_priority = self.__get_priority_conf(kw, 'msg_expire_senconds')
+        if msg_expire_senconds_priority != 0 and time.time() - msg_expire_senconds_priority > publish_time:
             self.logger.warning(f'消息发布时戳是 {publish_time} {kw["body"].get("publish_time_format", "")},距离现在 {round(time.time() - publish_time, 4)} 秒 ,'
-                                f'超过了指定的 {self._msg_expire_senconds} 秒，丢弃任务')
+                                f'超过了指定的 {msg_expire_senconds_priority} 秒，丢弃任务')
             self._confirm_consume(kw)
             return 0
         self.threadpool.submit(self._run, kw)
@@ -529,7 +569,7 @@ class ConcurrentModeDispatcher(LoggerMixin):
         elif self._concurrent_mode == 3:
             pool_type = CustomEventletPoolExecutor
             check_evenlet_monkey_patch()
-        self.consumer._threadpool = self.consumer._specify_threadpool if self.consumer._specify_threadpool else pool_type(self.consumer._threads_num + 1)  # 单独加一个检测消息数量和心跳的线程
+        self.consumer._threadpool = self.consumer._specify_threadpool if self.consumer._specify_threadpool else pool_type(self.consumer._threads_num)
         return self.consumer._threadpool
 
     def schedulal_task_with_no_block(self):
